@@ -52,11 +52,12 @@
 #include <string.h>
 #include <math.h>
 
+#include "aiffelements.h"
 #include "aiffparse.h"
 #include <gst/audio/audio.h>
 #include <gst/tag/tag.h>
 #include <gst/pbutils/descriptions.h>
-#include <gst/gst-i18n-plugin.h>
+#include <glib/gi18n-lib.h>
 
 GST_DEBUG_CATEGORY (aiffparse_debug);
 #define GST_CAT_DEFAULT (aiffparse_debug)
@@ -104,7 +105,10 @@ GST_STATIC_PAD_TEMPLATE ("src",
 #define MAX_BUFFER_SIZE 4096
 
 #define gst_aiff_parse_parent_class parent_class
-G_DEFINE_TYPE (GstAiffParse, gst_aiff_parse, GST_TYPE_ELEMENT);
+G_DEFINE_TYPE_WITH_CODE (GstAiffParse, gst_aiff_parse, GST_TYPE_ELEMENT,
+    GST_DEBUG_CATEGORY_INIT (aiffparse_debug, "aiffparse", 0, "AIFF parser"));
+GST_ELEMENT_REGISTER_DEFINE_WITH_CODE (aiffparse, "aiffparse", GST_RANK_PRIMARY,
+    GST_TYPE_AIFF_PARSE, aiff_element_init (plugin));
 
 static void
 gst_aiff_parse_class_init (GstAiffParseClass * klass)
@@ -824,7 +828,7 @@ gst_aiff_parse_create_caps (GstAiffParse * aiff)
 {
   GstCaps *caps = NULL;
   const gchar *format = NULL;
-  guint64 channel_mask;
+  guint64 channel_mask = 0;
 
   if (aiff->floating_point) {
     if (aiff->endianness == G_BIG_ENDIAN) {
@@ -862,42 +866,37 @@ gst_aiff_parse_create_caps (GstAiffParse * aiff)
         "rate", G_TYPE_INT, aiff->rate, NULL);
   }
 
-  if (aiff->channels > 2) {
-    GST_FIXME_OBJECT (aiff, "using fallback channel layout for %d channels",
-        aiff->channels);
+  /* based on AIFF-1.3.pdf */
+  switch (aiff->channels) {
+    case 1:
+      channel_mask = 0;
+      break;
+    case 2:
+      channel_mask = _P (FRONT_LEFT) | _P (FRONT_RIGHT);
+      break;
+    case 3:
+      channel_mask = _P (FRONT_LEFT) | _P (FRONT_RIGHT) | _P (FRONT_CENTER);
+      break;
+    case 4:
+      /* lists both this and 'quad' but doesn't say how to distinguish the two */
+      channel_mask =
+          _P (FRONT_LEFT) | _P (FRONT_RIGHT) | _P (REAR_LEFT) | _P (REAR_RIGHT);
+      break;
+    case 6:
+      channel_mask =
+          _P (FRONT_LEFT) | _P (FRONT_LEFT_OF_CENTER) | _P (FRONT_CENTER) |
+          _P (FRONT_RIGHT) | _P (FRONT_RIGHT_OF_CENTER) | _P (LFE1);
+      break;
+    default:
+      GST_FIXME_OBJECT (aiff, "using fallback channel layout for %d channels",
+          aiff->channels);
+      channel_mask = gst_audio_channel_get_fallback_mask (aiff->channels);
+      break;
+  }
 
-    /* based on AIFF-1.3.pdf */
-    switch (aiff->channels) {
-      case 1:
-        channel_mask = 0;
-        break;
-      case 2:
-        channel_mask = _P (FRONT_LEFT) | _P (FRONT_RIGHT);
-        break;
-      case 3:
-        channel_mask = _P (FRONT_LEFT) | _P (FRONT_RIGHT) | _P (FRONT_CENTER);
-        break;
-      case 4:
-        /* lists both this and 'quad' but doesn't say how to distinguish the two */
-        channel_mask =
-            _P (FRONT_LEFT) | _P (FRONT_RIGHT) | _P (REAR_LEFT) |
-            _P (REAR_RIGHT);
-        break;
-      case 6:
-        channel_mask =
-            _P (FRONT_LEFT) | _P (FRONT_LEFT_OF_CENTER) | _P (FRONT_CENTER) |
-            _P (FRONT_RIGHT) | _P (FRONT_RIGHT_OF_CENTER) | _P (LFE1);
-        break;
-      default:
-        channel_mask = gst_audio_channel_get_fallback_mask (aiff->channels);
-        break;
-    }
-
-
-    if (channel_mask != 0) {
-      gst_caps_set_simple (caps, "channel-mask", GST_TYPE_BITMASK, channel_mask,
-          NULL);
-    }
+  if (channel_mask != 0) {
+    gst_caps_set_simple (caps, "channel-mask", GST_TYPE_BITMASK, channel_mask,
+        NULL);
   }
 
   GST_DEBUG_OBJECT (aiff, "Created caps: %" GST_PTR_FORMAT, caps);
